class DF:
    '''
    Base class representing Distribution Function (RDF,ADF,Potential, etc.)
    It contains common properties and methods of distribution functions.     
        Properties:
            g[:,0:2]- table (numpy array) defining the function g[:,0]-keeps argument (r,angle) g[:,1]-keeps function's value            
            _ghist - histogram to accumalate statistics for RDF/ADF calculation.
            Name - Name of the DF
            Min,Max - range of distance/angle values where the function is defined
            type - type of the function: inter - intermolecular interaction related, intra - intramolecular interaction related function
            Npoints - Number of points in a table defining the  function
            Resol - resolution of the table defining the function                        
        Methods:
            
            Smooth - Smooth values of g[:,1] using Savitzky Golay 5 points smoothing procedure
            Trim(tolerance) - Cut function's (g[:,1]) left and right tails which have values smaller then <tolerance>
            Plot() - Plot the function, using matplotlib library
            Dump(filename)- Dump the object to a file <filename >using pickle library
            Save() - Write the function in a tabulated for to a text file.            
            Normalize - Calculate values of g[:,0:2] by normalizing accumulated histogram. 
    '''
    def __init__(self,Name=None,Min=None,Max=None,Npoints=None,
                 AtomTypes=None,AtomGroups=None,_AtomGroups_=None,
                 g=None,  Type=None, BondNumber=None, #Resol=None,
                 MolTypeName=None, Kind=None,Lines=None, MType=None):
        '''Construct the object of class DF: Either specify all the required properties, or just provide list of strings (Lines) specifying single RDF/potential section'''
        import numpy as np
               
        if Lines==None: # No lines provided - old-type constructor        
            self.Name=Name
            self.Kind=Kind
            self.Type=Type
            self.Npoints=Npoints        
            self.AtomGroups=AtomGroups
            self._AtomGroups_=_AtomGroups_
            self.AtomTypes=AtomTypes
            self.MolTypeName=MolTypeName
            self.BondNumber=BondNumber
            self.NGroups=len(AtomGroups) if AtomGroups!=None else None
            self.Min=Min
            self.Max=Max
            self.Resol=(self.Max-self.Min)/self.Npoints #if Resol==None else Resol
            if g!=None:
                self.g=g
            else:
                self.g=np.asarray([[self.Min+i*self.Resol+0.5*self.Resol,0.0] for i in xrange(0,self.Npoints)])
            self._Setup_()
        else:
            self.__InitFromLines__(Lines)
        self._ghist=np.zeros(self.Npoints)
        self.FirstNonZero=0
        self.LastNonZero=self.Npoints-1
        self._pairslist=[]        
        
    def _Setup_(self):
        '''
        Extra part to constructor to be overriden at each subclass
        '''
        pass;
    def __InitFromLines__(self,lines):
        '''
        InitFromLines(lines) - Reads section of a .rdf or .pot file provided in text lines and creates an object of specific type        
        '''
        import sys
        import numpy
                
        # Stage 1. Clean and check
        # 1.1 Clean lines from comments and leading spaces, convert all to uppercase
        def CleanAndUp(s):
            s=s.strip()
            if s.startswith('&'): s=s.upper()
            if '=' in s:
                ss=s.split('=')
                ss[0]=ss[0].upper()
                s='='.join(ss)
            return s
        lines = map(CleanAndUp, lines)
        lines = filter(lambda s: (s[0]!='!')&(s[0]!='#'),lines)

        # 2 Check for header tags, detect Kind
        knd='RDF'
        if lines[0]=='&'+knd:
            Kind=knd
            if lines.count('&'+knd)>1:
                sys.exit('too many &'+knd+' tags in lines')
            if lines.count('&END'+knd)!=1:        
                sys.exit('too many or too few &END'+knd+' tags in lines')
        knd='POTENTIAL'
        if lines[0]=='&'+knd:
            Kind=knd
            if lines.count('&'+knd)>1:
                sys.exit('too many &'+knd+' tags in lines')
            if lines.count('&END'+knd)!=1:        
                sys.exit('too many or too few &END'+knd+' tags in lines')               
        knd='INCLUDED'
        if lines.count('&'+knd)!=0:
            if lines.count('&'+knd)>1:
                sys.exit('too many &'+knd+' tags in lines')
            if lines.count('&END'+knd)!=1:        
                sys.exit('too many or too few &END'+knd+' tags in lines')               
        knd='TABLE'
        if lines.count('&'+knd)!=0:
            if lines.count('&'+knd)>1:
                sys.exit('too many &'+knd+' tags in lines')
            if lines.count('&END'+knd)!=1:        
                sys.exit('too many or too few &END'+knd+' tags in lines')        
        else:
                sys.exit('No TABLE section found in lines')                
        # 3 Read properties        
        def read_prop(knd,lines, must=True):    
            import re
            query=filter(lambda x: re.search('^'+knd+' *=',x),lines)
            if len(query)>0:
                return query[0].split('=')[1].strip()
            else:
                if must:
                    sys.exit('Error: No '+knd+' value provided')
                else:
                    return None        
        # 3.1 Name
        Name=read_prop('NAME',lines,must=True)
        # 3.2 Type
        Type=read_prop('TYPE',lines,must=True)
        if (not(Type in ('NB','B','A'))):
            sys.exit('Unknown type stated in lines. Type='+str(Type))
        # 3.3 Min Max
        Min=float(read_prop('MIN',lines,must=True))
        Max=float(read_prop('MAX',lines,must=True))
        # 3.4 NPoints
        NPoints=int(read_prop('NPOINTS',lines,must=True))
        # 3.5 AtomTypes if reading NB-rdf or potential
        AtomTypes=read_prop('ATOMTYPES',lines,must=(Type=='NB'))
        if Type=='NB': 
            AtomTypes=AtomTypes.replace(' ','').split(',')
        # 3.6 if bonded -rdf or potential -> read Pairs, Bond Number and MolecularType
        AtomGroups=None
        NGroups=None
        BondNumber=None
        MolTypeName=None
        if Type!='NB':
            if Type=='B':
                knd='PAIRS'
            elif Type=='A':
                knd='TRIPLETS'
            AtomGroups=read_prop(knd,lines,must=True).replace(' ','').split(',')
            NGroups=int(read_prop('N'+knd,lines,must=True))
            def cvrt(x):
                y=x.split('-')
                return map(int,y)
            AtomGroups=map(cvrt, AtomGroups)
            if (len(AtomGroups)!=NGroups):
                sys.exit('Actual number of atom groups {0:d} differs from stated {1:d}'.format(len(AtomGroups),NGroups))
            BondNumber=int(read_prop('BONDNUMBER',lines,must=True))
            MolTypeName=read_prop('MOLTYPE',lines,must=True)
        
        # 4. Read Table
        try:
            table=numpy.asarray(map(lambda x: x.split(),lines[lines.index('&TABLE')+1
            :lines.index('&ENDTABLE')]), dtype=numpy.float32)
        except:
            sys.exit('Can not read table - check the data')
            
        # 5 Set the object's properties
        self.Name=Name
        self.Type=Type
        self.Min=Min        
        self.Max=Max
        self.Npoints=NPoints        
        self.AtomTypes=AtomTypes #if AtomTypes!=None else None
        self.g=table
        self.AtomGroups=AtomGroups #if AtomGroups!=None else None
        self.NGroups=NGroups #if NGroups!=None else None
        self.Kind=Kind
        self.BondNumber=BondNumber #if BondNumber!=None else None
        self.MolTypeName=MolTypeName #if MolTypeName!=None else None
        self.Resol=(self.Max-self.Min)/self.Npoints
        
    def Normalize(self,volume):
        '''
        Normalize(volume) - Calculate values of g[:,0:2] by normalizing accumulated histogram. Intermolecular DFs are normalized over volume/(4*Pi*r**2*Npairs_counted). Intramolecular DFs are normalized over Npairs_counted. Only available with rdf-2.0.
        '''
        import numpy as np
        #Calculating total number of pairs in the RDF excepting pairs in point 0.
        npairs=sum(np.asarray(self._ghist))
        if self.Type=='NB':
            print "npairs",  npairs
            self.g[:,1]=np.asarray(self._ghist)*(volume/npairs)/(4*3.1415926536*self.Resol*(self.g[:,0]**2+self.Resol**2/12.0))
            self.g[0,1]=0.0
        elif self.Type in ('B','A'):
            self.g[:,1]=np.asarray(self._ghist)/(self.Resol*npairs)
    def Smooth(self):
        '''
        Smooth - Smooth values of g[:,1] using Savitzky Golay 5 points smoothing procedure
        '''    
        gol=self.g[:,1].copy()
        for i in xrange(2,self.Npoints-2):
            self.g[i,1]=gol[i]*0.486+gol[i-1]*0.343-gol[i-2]*0.086+gol[i+1]*0.343-gol[i+2]*0.086
            if self.g[i,1]<0: self.g[i,1]=0
    def Trim(self,tolerance):
        '''
        Trim(tolerance) - Cut function's (g[:,1]) left and right tails which have values smaller then <tolerance>
        '''
        import numpy as np
        imax=self.g[:,1].argmax() #position of the maximum
        i=0
        while ((self.g[i,1]<tolerance) and (i<imax)): i+=1
        self.g[0:i,1]=0.0
        self.FirstNonZero=i
        i=self.Npoints-1
        while ((self.g[i,1]<tolerance) and (i>imax)): i-=1
        if i!=(self.Npoints-1):
            self.g[i:,1]=0.0
            self.LastNonZero=i-1
        if self.Type in ('B','A'):
            self.g[:,1]=self.g[:,1]/np.trapz(self.g[:,1],self.g[:,0])
        self.Npoints=self.LastNonZero-self.FirstNonZero+1
        self.Min=self.g[self.FirstNonZero,0]-self.Resol*0.5
        self.Max=self.g[self.LastNonZero,0]+self.Resol*0.5
    def Plot(self,hardcopy=False,linetype='',nolegendintra=False,hidden=False, noshow=False, linewidth=1,legend=None):
        '''
        Plot() - Plot the function, using matplotlib library
        hardcopy - If to save the copy of plot to eps-file
        linetype - define type of lines to use (linetype='.' use dots, linetype='- 'use solid lines)
        nolegendintra - turn off legend on plot with intramolecular DFs. Usually it is usefull then legend overlaps with lines on the plot. 
        legend - legend for the function
        '''
        import matplotlib.pyplot as plt
        ttl=self.Name+' (%s)' %(self.Type)
        if (nolegendintra) and (self.Type in ('B','A')):
            plt.plot(self.g[self.FirstNonZero:self.LastNonZero+1,0],self.g[self.FirstNonZero:self.LastNonZero+1,1],linetype,linewidth=linewidth)
        else:
            if legend!=None:
                plt.plot(self.g[self.FirstNonZero:self.LastNonZero+1,0],self.g[self.FirstNonZero:self.LastNonZero+1,1],linetype,label=legend,linewidth=linewidth)
            else:
                plt.plot(self.g[self.FirstNonZero:self.LastNonZero+1,0],self.g[self.FirstNonZero:self.LastNonZero+1,1],linetype,label=self.Name+' (%s)' %(self.Type),linewidth=linewidth)
        if (not hidden) or hardcopy:
            plt.savefig(ttl.replace(' ','.')+'.eps')            
        if not noshow:
            plt.show()
             
    def Dump(self,filename):
        '''
        Dump(filename)- Dump the object to a file <filename >using pickle library
        '''
        import pickle 
        f=open(filename,'ab')
        pickle.dump(self,f)
        f.close()
    def Save(self,filename=''):
        '''
        Save() - Write the function in a tabulated for to a text file.
        Filename is definead as (self.Name+self.Type+filename).replace(' ','.')
        '''
        f=open('{0}.{1}.{2}.dat'.format(self.Type,self.Name,filename).replace(' ','.'),'w')
        f.write('# R      '+self.Name+self.Type+'\n')
        for i in xrange(0,len(self.g[:,0])):
            f.write('%9.5f  %9.5f \n' %(self.g[i,0],self.g[i,1]))
        f.close

class RDF(DF):
    pass
class Pot(DF):
    pass
class DF_NB(DF):
    def _Write(self,ofile, Split=False, inc_file=None, Trim=False):
        '''
        Write the DF-object to the file ofile. If Split=True, the table is written to a separate include-file.
        The include file name can be specified in inc_file, otherwise it is generated automatically.
        if Trim = True, the left zero-sholder will be trimmed (mmostly needed for RDF).
        '''
        ofile.write('&{0} \n'.format(self.Kind))
        ofile.write('Name={0} \n'.format(self.Name))
        ofile.write('Type={0} \n'.format(self.Type))
        ofile.write('Min={0:8.4f} \nMax={1:8.4f} \n'.format(self.Min,self.Max))
        ofile.write('Npoints={0:d} \n'.format(self.Npoints))                    

        ofile.write("AtomTypes={0}\n".format(', '.join(self.AtomTypes)))                
        # Prepare list of lines representing data-table
        if Trim:
            TableLines=['{0:11.7f}  {1:11.7f} \n'.format(float(self.g[i][0]),float(self.g[i][1])) for i in xrange(0, len(self.g)) if (self.g[i][0]>self.Min) and (self.g[i][0]<self.Max)]            
        else:
            TableLines=['{0:11.7f}  {1:11.7f} \n'.format(float(self.g[i][0]),float(self.g[i][1])) for i in xrange(0, len(self.g))]        
        TableLines=['&Table\n',]+TableLines+['&EndTable\n']
       
        if (Split): # if we write table to the include file
            if inc_file==None: # If no name provided - create one
                ofilename=ofile.name
                inc_file="{0}.{1}.{2}.inc".format(ofilename,self.Type,'-'.join(self.AtomTypes))                		
            ofile.write('&Include{0}={1}\n'.format(self.Kind,inc_file))
            oinc_file=open(inc_file,'w')
            oinc_file.write('&Included{0} \n'.format(self.Kind))
            oinc_file.write('Name={0} \n'.format(self.Name))
            oinc_file.write('Type={0} \n'.format(self.Type))
            oinc_file.write('Min={0:8.4f} \nMax={1:8.4f}\n'.format(self.Min,self.Max))
            oinc_file.write('Npoints={0:d} \n'.format(self.Npoints))
            oinc_file.write("AtomTypes={}\n".format(', '.join(self.AtomTypes)))
            oinc_file.writelines(TableLines)
            oinc_file.write('&ENDIncluded{0} \n'.format(self.Kind))
            oinc_file.close()
        else: # No split
            ofile.writelines(TableLines)
        ofile.write('&END{0} \n'.format(self.Kind))
        return
class DF_B(DF):
    def _Write(self,ofile,Split=False, inc_file=None):
        ofile.write('&{0} \n'.format(self.Kind))
        ofile.write('Name={0} \n'.format(self.Name))
        ofile.write('Type={0} \n'.format(self.Type))
        ofile.write('Min={0:8.4f} \nMax={1:8.4f} \n'.format(self.Min,self.Max))
        ofile.write('Npoints={0:d} \n'.format(self.Npoints))                    

        ofile.write("MolType = {}\n".format(self.MolTypeName))
        ofile.write("NPairs={0:d}\n".format(len(self.AtomGroups)))
        ofile.write("Pairs={}\n".format(', '.join(['-'.join(map(str,self.AtomGroups[i][:])) for i in xrange(0,len(self.AtomGroups))])))   
        ofile.write("Bondnumber={0:d}\n".format(self.BondNumber))     

        # Prepare list of lines representing data-table
        TableLines=['{0:11.7f}  {1:11.7f} \n'.format(float(self.g[i][0]),float(self.g[i][1])) for i in xrange(self.FirstNonZero, self.LastNonZero+1)]#xrange(0,len(self.g))]
        TableLines=['&Table\n',]+TableLines+['&EndTable\n']

        
        if (Split): # if we write table to the include file
            if inc_file==None: # If no name provided - create one
                ofilename=ofile.name
                inc_file="{0}.{1}.{2}.{3}.inc".format(ofilename,self.Type,self.MolTypeName,self.BondNumber)
            ofile.write('&Include{0}={1}\n'.format(self.Kind,inc_file))
            oinc_file=open(inc_file,'w')
            oinc_file.write('&Included{0} \n'.format(self.Kind))
            oinc_file.write('Name={0} \n'.format(self.Name))
            oinc_file.write('Type={0} \n'.format(self.Type))
            oinc_file.write('Min={0:8.4f} \nMax={1:8.4f}\n'.format(self.Min,self.Max))
            oinc_file.write('Npoints={0:d} \n'.format(self.Npoints))
            oinc_file.write("MolType = {}\n".format(self.MolTypeName))
            oinc_file.write("NPairs={0:d}\n".format(len(self.AtomGroups)))
            oinc_file.write("Pairs={}\n".format(', '.join(['-'.join(map(str,self.AtomGroups[i][:])) for i in xrange(0,len(self.AtomGroups))])))   
            oinc_file.write("Bondnumber={0:d}\n".format(self.BondNumber))                             
            oinc_file.writelines(TableLines)
            oinc_file.write('&ENDIncluded{0} \n'.format(self.Kind))
            oinc_file.close()
        else: # No split
            ofile.writelines(TableLines)
        ofile.write('&END{0} \n'.format(self.Kind))
        return
class DF_A(DF):
    def _Write(self,ofile,Split=False, inc_file=None):
        ofile.write('&{0} \n'.format(self.Kind))
        ofile.write('Name={0} \n'.format(self.Name))
        ofile.write('Type={0} \n'.format(self.Type))
        ofile.write('Min={0:8.4f} \nMax={1:8.4f} \n'.format(self.Min,self.Max))
        ofile.write('Npoints={0:d} \n'.format(self.Npoints))                    

        ofile.write("MolType = {}\n".format(self.MolTypeName))
        ofile.write("Bondnumber={0:d}\n".format(self.BondNumber))
        ofile.write("NTriplets={0:d}\n".format(len(self.AtomGroups)))
        ofile.write("Triplets={}\n".format(', '.join(['-'.join(map(str,self.AtomGroups[i][:])) for i in xrange(0,len(self.AtomGroups))])))           

       # Prepare list of lines representing data-table
        TableLines=['{0:11.7f}  {1:11.7f} \n'.format(float(self.g[i][0]),float(self.g[i][1])) for i in xrange(self.FirstNonZero, self.LastNonZero+1)]
        TableLines=['&Table\n',]+TableLines+['&EndTable\n']
        
        if (Split): # if we write table to the include file
            if inc_file==None: # If no name provided - create one
                ofilename=ofile.name
                inc_file="{0}.{1}.{2}.{3}.inc".format(ofilename,self.Type,self.MolTypeName,self.BondNumber)
            ofile.write('&Include{0}={1}\n'.format(self.Kind,inc_file))
            oinc_file=open(inc_file,'w')
            oinc_file.write('&Included{0} \n'.format(self.Kind))
            oinc_file.write('Name={0} \n'.format(self.Name))
            oinc_file.write('Type={0} \n'.format(self.Type))
            oinc_file.write('Min={0:8.4f} \nMax={1:8.4f}\n'.format(self.Min,self.Max))
            oinc_file.write('Npoints={0:d} \n'.format(self.Npoints))
            oinc_file.write("MolType = {}\n".format(self.MolTypeName))
            oinc_file.write("BondNumber={0:d}\n".format(self.BondNumber))
            oinc_file.write("NTriplets={0:d}\n".format(len(self.AtomGroups)))
            oinc_file.write("Triplets={}\n".format(', '.join(['-'.join(map(str,self.AtomGroups[i][:])) for i in xrange(0,len(self.AtomGroups))])))               
            oinc_file.writelines(TableLines)
            oinc_file.write('&ENDIncluded{0} \n'.format(self.Kind))
            oinc_file.close()
        else: # No split
            ofile.writelines(TableLines)
        ofile.write('&END{0} \n'.format(self.Kind))
        return
class RDF_A(RDF,DF_A):
    pass
class RDF_B(RDF,DF_B):
    pass
class RDF_NB(RDF,DF_NB):
    def Trim(self,tolerance):
        import DF
        DF.RDF.Trim(self,tolerance)
    pass
    def _Write(self,ofile, Split=False, inc_file=None, Trim=True):
        import DF
        DF.DF_NB._Write(self,ofile, Split=Split, inc_file=inc_file, Trim=Trim)
class Pot(DF):
    '''Dummy class inherited from base class DF and parent for specific potential types, which have diferrent implementations of Export2Gromacs, and PotPressCorr method'''
    def Export2Gromacs(self, npoints=18000, Umax=2000, Rmaxtable=25, noplot=False, hardcopy=True, figsize=(14,8), dpi=120, sigma=0.0, zeroforce=False, interpol=True, radians=False, ofilename=''):
        '''Dummy method: to be defined in inheritor class'''
        pass
    def _GaussInterpol_(self,R0,U0,R_ie,sigma=0.0):
        '''Auxilary subroutine performing Gaussian smoothing-interpolation of given potential. U0,R0 - given set of points describing potential, R_ie-points defining resuling potential. Sigma defines gaussian's decay spe ed. default vauie is half distance between points in R0'''
        import numpy as np
        U_ie=np.zeros(len(R_ie))
        if sigma==0.0:
            sigma=(R0[1]-R0[0])
        for i in xrange(0,len(R_ie)):
            r=R_ie[i]
            w=np.exp(-np.square(r-R0)/(2*sigma**2))
            U_ie[i]=np.dot(U0,w)/np.sum(w)            
        return U_ie
    def PotPressCorr(self,U_corr0):
        ''' Dummy method, to be defined in inheritor. Suppose to bring linear correction term to the potential'''
        pass
class Pot_A(Pot,DF_A):
    def Export2Gromacs(self,npoints=1800, Umax=2000,Rmaxtable=180.0, noplot=False, hardcopy=True, figsize=(14,7.5), dpi=120, sigma=0.0, zeroforce=False, interpol=True, Ushape=True, radians=False, ofilename=None):
        import matplotlib.pyplot as plt
        import numpy as np        
        if self.BondNumber==None:
            print '{0} is not exported, since the bond number is not known'.format(self.Name)
            return
        if ofilename==None:
            ofilename='table_'+self.Name+'.'+self.Type+'.xvg'

        print '{0:<50s} -> {1:<s}'.format(self.Name,ofilename)
        res=self.Resol       
        if radians:
            R0=self.g[:,0]*180.0/np.pi
            res=res*180.0/np.pi
        else:
            R0=self.g[:,0]
        U0=self.g[:,1]-np.min(self.g[:,1])         
        sigma=(R0[1]-R0[0])*sigma
        if U0[-1]==0.0:
            Ushape=False
        if not interpol:            
            npoints=int(round(2*(Rmaxtable-0.0)/res))
        # Generate sets of R-points
        R_ie=np.linspace(0,Rmaxtable,npoints+1)
        R_rtail_ie=R_ie[np.where(R_ie>R0[-1])]
        R_ltail_ie=R_ie[np.where(R_ie<R0[0])]
        tmp=R_ie[np.where(R_ie<=R0[-1])]
        R_c_ie=tmp[np.where(tmp>=R0[0])]
        # Left tail iextrapolation - linear fit
        ltailpoints=int(round((R0[0]/res-0.5)))
        R_ltail=np.append(0,np.linspace(res/2,R0[0],ltailpoints+1))
        k_ltail=(np.diff(U0[0:2])/np.diff(R0[0:2]))        
        a=((U0[0]-Umax)-k_ltail*R0[0])/(-R0[0]**2)
        b=k_ltail-2*R0[0]*a        
        U_ltail=np.square(R_ltail)*a+R_ltail*b+Umax
        U_ltail=U_ltail[0:len(U_ltail)-1]
        R_ltail=R_ltail[0:len(R_ltail)-1]
        #Right tail ie-polation        
        rtailpoints=int(round((Rmaxtable-R0[-1])/res-0.5))
        R_rtail=np.append(np.linspace(R0[-1],Rmaxtable-res/2,rtailpoints+1)[1:],Rmaxtable)
        if Ushape:
            k_rtail=(np.diff(U0[self.Npoints-2:self.Npoints])/np.diff(R0[self.Npoints-2:self.Npoints]))
            U_rtail=k_rtail*(R_rtail-R0[-1])+U0[-1]
        else:
            U_rtail=np.exp(-100.0*(R_rtail-R0[-1])/(Rmaxtable-R0[-1]))*U0[-1]
        U=np.append(U_ltail,np.append(U0,U_rtail))
        R=np.append(R_ltail,np.append(R0,R_rtail))
        U_ie=self._GaussInterpol_(R,U,R_ie,sigma=sigma)
        F_ie=np.append(-np.diff(U_ie)/np.diff(R_ie),0.0)        
        F=F_ie[len(R_ltail_ie):len(F_ie)-len(R_rtail_ie)]        
        if not interpol:
            for i in xrange(0,len(U0)):
                U_ie[len(R_ltail_ie)+i*2]=U0[i]
        if not noplot:
            fig=plt.figure(figsize=figsize,dpi=dpi)
            ax1=fig.add_subplot(211)
            plt.title(self.Name)
            ax3=fig.add_subplot(223)
            ax5=fig.add_subplot(224)
            ax1.plot(R,U,'y.',label='Extrapolated tails')            
            ax1.plot(R0,U0,'r.',label='Original potential')            
            ax1.plot(R_ie,U_ie,'b-',label='Interpolated potential')       
            ax1.legend(('Extrapolated tails','Original potential','Interpolated potential'),loc=0,fancybox=False)             
            ax1.set_xlim(2*R[ltailpoints+1]-R[ltailpoints+2],2*R[-1-rtailpoints]-R[-2-rtailpoints])
            ax1.set_ylim(min(U[ltailpoints:len(U)-rtailpoints])*0.8,max(U[ltailpoints:len(U)-rtailpoints])*1.2)                             
            ax2=ax1.twinx()
            ax2.plot(R_ie,F_ie,'g-',label='Force from IE-polated pot')
            ax2.set_xlim(2*R[ltailpoints+1]-R[ltailpoints+2],2*R[-1-rtailpoints]-R[-2-rtailpoints])
            ax2.set_ylim(min(F),max(F))                             
            ax1.set_xlabel('Angle, deg')
            ax1.set_ylabel('Potential kJ/mol')
            ax2.set_ylabel('Force kJ/(mol*deg)')                                    
            ax3.plot(R,U,'y.',label='Extrapolated tails')            
            ax3.plot(R0,U0,'r.',label='Original potential')            
            ax3.plot(R_ie,U_ie,'b-',label='Interpolated potential')
            ax3.set_xlabel('Angle, deg')
            ax3.set_ylabel('Potential kJ/mol')
            ax5.plot(R,U,'yo',label='Extrapolated tails') 
            ax5.plot(R,U,'ro',label='Original potential')            
            ax5.plot(R_ie,U_ie,'b-',label='Interpolated potential')
            ax5.set_xlabel('Angle, deg')
            ax5.set_ylabel('Potential kJ/mol')
            ax5.set_xlim(R0[-20],R0[-1])
            ax5.set_ylim(min(U0[-20:]),max(U0[-20:]))                        
            if hardcopy:
                plt.savefig(ofilename+'.png')
            plt.show()
        ofile=open(ofilename,'w')
        if zeroforce:
            F=F*0.0
            F_ie=F_ie*0.0
        for i in xrange(0,len(U_ie)):
            ofile.write('%12.6f   %12.6f %12.6f \n' %(R_ie[i],U_ie[i],F_ie[i]))
        ofile.close()
    def PotPressCorr(self,U_corr0):
        ''' Dummy method, only print information message that nothing was done.'''
        print "No correction was applied to angle bending potential ",str(self.Name)
        pass
class Pot_B(Pot,DF_B):
    def Export2Gromacs(self,npoints=2500,Umax=6000,Rmaxtable=2.5,ofilename=None,noplot=False,hardcopy=True,figsize=(14,7.5),dpi=120,sigma=0.0,zeroforce=False,interpol=True):
        import matplotlib.pyplot as plt        
        import numpy as np
        if self.BondNumber==None:
            print '{0} is not exported, since the bond number is not known'.format(self.Name)
            return
        if ofilename==None:    
            ofilename='table_'+self.Name+'.'+self.Type+'.xvg'                
        print '{0:<50s} -> {1:<s}'.format(self.Name,ofilename)
        R0=self.g[:,0]*0.1
        U0=self.g[:,1]-np.min(self.g[:,1])        
        sigma=(R0[1]-R0[0])*sigma
        res=self.Resol*0.1
        if not interpol:
            npoints=int(round(2*(Rmaxtable-0.0)/res))        
        # Generate sets of R-points
        R_ie=np.linspace(0,Rmaxtable,npoints+1)        
        R_rtail_ie=R_ie[np.where(R_ie>R0[-1])]
        R_ltail_ie=R_ie[np.where(R_ie<R0[0])]
        tmp=R_ie[np.where(R_ie<=R0[-1])]
        R_c_ie=tmp[np.where(tmp>=R0[0])]
        # Left tail iextrapolation
        ltailpoints=int(round((R0[0]/res-0.5)))
        R_ltail=np.append(0,np.linspace(res/2,R0[0],ltailpoints+1))
        k_ltail=(np.diff(U0[0:2])/np.diff(R0[0:2]))
        a=((U0[0]-Umax)-k_ltail*R0[0])/(-R0[0]**2)
        b=k_ltail-2*R0[0]*a        
        U_ltail=np.square(R_ltail)*a+R_ltail*b+Umax
        U_ltail=U_ltail[0:len(U_ltail)-1]
        R_ltail=R_ltail[0:len(R_ltail)-1]
        #Right tail ie-polation
        rtailpoints=int(round((Rmaxtable-R0[-1])/res-0.5))
        R_rtail=np.append(np.linspace(R0[-1],Rmaxtable-res/2,rtailpoints+1)[1:],Rmaxtable)
        k_rtail=(np.diff(U0[self.Npoints-2:self.Npoints])/np.diff(R0[self.Npoints-2:self.Npoints]))
        a=((U0[-1]-Umax)-k_rtail*R0[-1])/(-R0[-1]**2)
        b=k_rtail-2*R0[-1]*a        
        U_rtail=np.square(R_rtail)*a+R_rtail*b+Umax
               
        U=np.append(U_ltail,np.append(U0,U_rtail))
        R=np.append(R_ltail,np.append(R0,R_rtail))
        U_ie=self._GaussInterpol_(R,U,R_ie,sigma=sigma)
        if not interpol:
            for i in xrange(0,len(U0)):
                U_ie[len(R_ltail_ie)+i*2]=U0[i]
        F_ie=np.append(-np.diff(U_ie)/np.diff(R_ie),0.0)        
        F=-np.append(np.diff(U)/np.diff(R),[0.0,])       
        if not noplot:
            fig=plt.figure(figsize=figsize,dpi=dpi)
            ax1=fig.add_subplot(211)
            plt.title(self.Name)
            ax3=fig.add_subplot(212)
            ax1.plot(R,U,'y.')
            ax1.plot(R0,U0,'r.')    
            ax1.plot(R_ie,U_ie,'b-')                                 
            ax1.set_xlim(R[ltailpoints-2],R[len(R)-rtailpoints+2])
            ax1.set_ylim(min(U[ltailpoints:len(U)-rtailpoints])*0.8,max(U[ltailpoints:len(U)-rtailpoints])*1.2)                       
            ax2=ax1.twinx()
            ax2.plot(R_ie,F_ie,'g-',label='Force from IE-polated pot')            
            ax2.set_xlim(2*R[ltailpoints+1]-R[ltailpoints+2],2*R[-1-rtailpoints]-R[-2-rtailpoints])
            ax2.set_ylim(min(F[ltailpoints+1:len(F)-rtailpoints]),max(F[ltailpoints+1:len(F)-rtailpoints]))
            ax1.set_xlabel('R, nm')
            ax1.set_ylabel('Potential kJ/mol')
            ax2.set_ylabel('Force kJ/(mol*nm)')            
            ax3.plot(R,U,'y.',label='Extrapolated tails')            
            ax3.plot(R0,U0,'r.',label='Original potential')            
            ax3.plot(R_ie,U_ie,'b-',label='Interpolated potential')
            ax3.set_xlabel('R, nm')
            ax3.set_ylabel('Potential kJ/mol')        
            ax3.legend(('Extrapolated tails','Original potential','Interpolated potential'),loc=0,fancybox=False)            
            ax4=ax3.twinx()
            ax4.plot(R_ie,F_ie,'g-',label='Force')                        
            ax4.legend(loc=7)
            if hardcopy:
                plt.savefig(ofilename+'.png')
            plt.show()
        ofile=open(ofilename,'w')
        if zeroforce:
            F=F*0.0
            F_ie=F_ie*0.0
        for i in xrange(0,len(U_ie)):
            ofile.write('%12.6f   %12.6f %12.6f \n' %(R_ie[i],U_ie[i],F_ie[i]))
        ofile.close()
    def PotPressCorr(self,U_corr0):
        ''' Dummy method, only print information message that nothing was done.'''
        print "No correction was applied to bond potential ",str(self.Name)
        pass
class Pot_NB(Pot, DF_NB):    
    def __init__(self,Name=None,Min=None,Max=None,Npoints=None,AtomTypes=None,AtomGroups=None,
                 g=None,  Type=None, BondNumber=None, #Resol=None,
                 MolTypeName=None, Kind=None,Lines=None, Ucut=None):
        import numpy as np
        Pot.__init__(self,Name=Name,Min=Min,Max=Max,Npoints=Npoints,AtomTypes=AtomTypes,AtomGroups=AtomGroups,
                 g=g,  Type=Type, BondNumber=BondNumber, #Resol=None,
                 MolTypeName=MolTypeName, Kind=Kind,Lines=Lines)        
        if Ucut!=None:
            self.g=self.g[np.where(self.g[:,1]<Ucut)]
            self.Min=np.min(self.g[:,0])-self.Resol/2.0
            self.Max=np.max(self.g[:,0])+self.Resol/2.0
            self.Npoints=len(self.g)
        self._Setup_()
    def _Setup_(self,Ucut=None):
        '''
        Initialize the property qq in the NB-potenital. It is needed for potential decomposition on short range and electrostatic parts.
        '''
        self.qq=None
    def Export2Gromacs(self,npoints=2500, Umax=6000,Rmaxtable=2.5,noplot=False,hardcopy=True,figsize=(14,7.5),dpi=120, sigma=0.5, zeroforce=False, interpol=True,ofilename=None):
        import matplotlib.pyplot as plt
        import numpy as np

        if ofilename==None:
            ofilename='table_'+self.Name+'.xvg'                    # Convert Potential table to Gromacs units (kJ/mol, nm)
        print '{0:<50s} -> {1:<s}'.format(self.Name,ofilename)            
        R0=self.g[:,0]*0.1
        sigma=(R0[1]-R0[0])*sigma
        U0=self.g[:,1]                
        res=self.Resol*0.1
        # Generate sets of R-points        
        if not interpol:
            npoints=int(round(2*(Rmaxtable-0.0)/res))
        R_ie=np.linspace(0,Rmaxtable,npoints+1)
        res_ie=R_ie[1]-R_ie[0]        
        R_rtail_ie=R_ie[np.where((R_ie-R0[-1])>res_ie/2)]
        R_ltail_ie=R_ie[np.where((R0[0]-R_ie)>res_ie/2)]
        tmp=R_ie[np.where(R_ie<=R0[-1])]
        R_c_ie=tmp[np.where(tmp>=R0[0])]
        # Left tail extrapolation
        ltailpoints=int(round((R0[0]/res-0.5)))
        R_ltail=np.append(0,np.linspace(res/2,R0[0],ltailpoints+1))
        k_ltail=(np.diff(U0[0:2])/np.diff(R0[0:2]))
        a=((U0[0]-Umax)-k_ltail*R0[0])/(-R0[0]**2)
        b=k_ltail-2*R0[0]*a        
        U_ltail=np.square(R_ltail)*a+R_ltail*b+Umax
        U_ltail=U_ltail[0:len(U_ltail)-1]
        R_ltail=R_ltail[0:len(R_ltail)-1]
        #Right tail extrapolation       
        rtailpoints=int(round((Rmaxtable-R0[-1])/res-0.5))
        R_rtail=np.append(np.linspace(R0[-1],Rmaxtable-res/2,rtailpoints+1)[1:],Rmaxtable)
        U_rtail=np.exp(-5.0*(R_rtail-R0[-1])/(Rmaxtable-R0[-1]))*U0[-1]

        #Right tail extrapolation - experimental
        U_rtail=0.0*R_rtail
       # Join all parts together
        U=np.append(U_ltail,np.append(U0,U_rtail))
        R=np.append(R_ltail,np.append(R0,R_rtail))
        # Interpolation and force calculation
        U_ie=self._GaussInterpol_(R,U,R_ie,sigma=sigma)
        F_ie=np.append(-np.diff(U_ie)/np.diff(R_ie),0.0)        
        F=F_ie[len(R_ltail_ie):len(F_ie)-len(R_rtail_ie)]
        if not interpol:
            for i in xrange(0,len(U0)):
                U_ie[len(R_ltail_ie)+i*2]=U0[i]
        if not noplot:
            fig=plt.figure(figsize=figsize,dpi=dpi)
            ax1=fig.add_subplot(211)
            plt.title(self.Name)
            ax3=fig.add_subplot(212)            
            ax1.plot(R,U,'y.')
            ax1.plot(R0,U0,'r.')    
            ax1.plot(R_ie,U_ie,'b-')                                           
            ax2=ax1.twinx()
            ax2.plot(R_ie,F_ie,'g-',label='Force')
            ax1.set_xlabel('R, nm')
            ax1.set_ylabel('Potential kJ/mol')            
            ax2.set_ylabel('Force kJ/(mol*nm)')            
            ax1.set_xlim(R[ltailpoints-1],R[+2-rtailpoints])
            y_low_lim=min(U[ltailpoints-1:len(U)-rtailpoints+2])
            y_upp_lim=max(U[ltailpoints-2:len(U)-rtailpoints+2])
            y_margin=(y_upp_lim-y_low_lim)*0.05
            ax1.set_ylim(y_low_lim-y_margin,y_upp_lim+y_margin)
            ax2.set_ylim(np.min(F[np.argmin(U[ltailpoints:]):]),np.max(F[np.argmin(U[ltailpoints:]):]))                     
            ax3.plot(R,U,'yo',label='Extrapolated tails')            
            ax3.plot(R0,U0,'ro',label='Original potential')            
            ax3.plot(R_ie,U_ie,'b-',label='Interpolated potential')
            ax3.legend(('Extrapolated tails','Original potential','Interpolated potential'),loc=0,fancybox=False)            
            ax3.set_xlabel('R, nm')
            ax3.set_ylabel('Potential kJ/mol')
            ax4=ax3.twinx()
            ax4.plot(R_ie,F_ie,'g-',label='Force')            
            ax4.legend(loc=7)
            if hardcopy:
                plt.savefig(ofilename+'.png')
            plt.show()
        ofile=open(ofilename,'w')
        if zeroforce:
            F=F*0.0
            F_ie=F_ie*0.0
        for i in xrange(0,len(U_ie)):
            ofile.write('{0:12.6f}   {1:12.6f} {2:12.6f} {3:12.6f} {4:12.6f} {5:12.6f} {6:12.6f} \n'.format(R_ie[i],0,0,U_ie[i],F_ie[i],0,0))
        ofile.close()          
    def GetWeight(self,eps,eps_orig,r1):
        '''Get weight of the potentials tail in range R1:Rmax (A). The weight is defined as absolute value of integral of r^2*U(r)dr taken from R1 to Rmax
        eps_orig is dielectric permittivity used for IMC calculation, while eps is dielectric permittivity used for current splitting of the potential on short-range ang electrostatic parts.        
        '''
        import numpy as np
        rmax=self.Max
        if r1<self.Min:
            print "Error: r1 is less than r_min"
            return
        U_sr_new=self.g[:,1]+((self.qq*1.602e-19*1.602e-19*6.022e23)/(4.0*3.1416*8.85e-12*1.0e-10*1.0e3))*(1.0/eps_orig-1.0/eps)*(1.0/self.g[:,0])
        r2=self.g[:,0]**2        
        rng=np.where(self.g[:,0]>=r1)
        weight=np.abs(np.sum(U_sr_new[rng]*r2[rng]*self.Resol))
        U_sr_new=self.g[:,1]*self.g[:,0]**2
        weight2=np.sum(U_sr_new[rng]*self.Resol)+0.5*(rmax**2-r1**2)*((self.qq*1.602e-19*1.602e-19*6.022e23)/(4.0*3.1416*8.85e-12*1.0e-10*1.0e3))*(1.0/eps_orig-1.0/eps)
        return weight
    def GetCharges(self,mcmfile):
        import MolType
        '''
        Load charges to the atom types from molecular descriptions files (*.mcm)        
        '''
        if not isinstance(mcmfile,type([])):
            mcmfile=[mcmfile]
        MolTypes=[MolType.MolType(mcm) for mcm in mcmfile]
        Q1=[A.Charge for MT in MolTypes for A in MT.Atoms if A.Type in self.AtomTypes[0]]
        if len(Q1)<1:
            print 'Error: Can not detect atoms with type {0} in files {1}\nCheck your mcm files'.format(
            self.AtomTypes[0],mcmfile)
            return
        Q2=[A.Charge for MT in MolTypes for A in MT.Atoms if A.Type in self.AtomTypes[1]]
        if len(Q1)<1:
            print 'Error: Can not detect atoms with type {0} in files {1}\nCheck your mcm files'.format(
            self.AtomTypes[1],mcmfile)
            return
        if len(Q1)>1 or len(Q2)>1:
            print 'Warning: More than one atom is detected for the atom type. The charge value is taken from the first record.'
        print '{0}, Charge {1}  <-->  {2}, Charge {3}'.format(self.AtomTypes[0],Q1[0],self.AtomTypes[1],Q2[0])
        self.qq=Q1[0]*Q2[0]
        return
    def PotPressCorr(self,U_corr0):
        '''Adding decaying linear correction, which suppose to improve pressure of represented system. 
            Correction term is linear and has value of U_corr0 at point r=0, and value of 0 at r=rmax'''
        print "Applying correction to intermolecular potential ",str(self.Name),' Ucorr(0)=',str(U_corr0)        
        self.g[:,1]+=U_corr0-(float(U_corr0)/self.Max)*self.g[:,0]
    
class RDFref(RDF):
    '''Class of reference RDFs. It differs from parent only in line thickness in plot() method'''
    def Plot(self,hardcopy=False,linetype='',nolegendintra=False,hidden=False, noshow=False, linewidth=2,legend=None):
        DF.Plot(self,hardcopy=hardcopy,linetype=linetype,nolegendintra=nolegendintra,hidden=hidden, noshow=noshow, linewidth=linewidth, legend=legend)         
            
class RDFref_NB(RDFref,DF_NB):            
    pass
class RDFref_B(RDFref,DF_B):            
    pass
class RDFref_A(RDFref,DF_A):            
    pass
