class DFset:
    '''
    Class representing a set of Distribution Functions (RDF,Potential,Potential correction, etc.)

        Properties:
            Name - Name of the set (typically shall describe how this set arises) \n            
            Kind - Kind of functions collected in the set: RDF or POTENTIAL
            NTypes - Number of different atomic types used in the set
            Types - Names of the atomic types
            Min, Max - Range of distance values for non-bonded interaction functions
            Npoints - Number of points in non-bonded interaction functions
            DFs - List of functions (all functions in the set)
            DFs_NB - List of non-bonded interaction functions 
            DFs_B - List of pairwise bond interaction functions
            DFs_A - List of angle-bending bond interaction functions
            
        Methods:
            DFset() - Construct the object from provided rdf/pot file (recommended way) or from the provided parameters (old fashion)
            Write() - Write the set of functions to the file.
            Plot() - Plot the set of functions
    '''
#            InitFromFile() - Creates a set of Distribution Functions (RDF,ADF,Potential) from the given rdf or potential file (MagiC 2.0 format)    

    def __init__(self, File=None,Kind=None, Name=None, Types=None, Min=None, Max=None, Npoints=None, DFs=None, Ucut=None):
        if File!=None:
            self.__InitFromFile__(File, Ucut=Ucut)
        else:
            self.Kind=Kind
            if Types!=None:
                self.Types=Types
            else:
                def uniq(seq): 
                    # order preserving
                   noDupes = []
                   [noDupes.append(i) for i in seq if not noDupes.count(i)]
                   return noDupes
                self.Types=uniq(list([at for DF in DFs if DF.Type=='NB' for at in DF.AtomTypes]))                        
            self.NTypes=len(self.Types)
            self.Min=Min
            self.Max=Max
            self.Npoints=Npoints
            self.DFs=DFs
            if Name!=None:
                self.Name=Name
        self.DFs_NB=[x for x in self.DFs if x.Type=='NB']        
        self.DFs_B=[x for x in self.DFs if x.Type=='B']
        self.DFs_A=[x for x in self.DFs if x.Type=='A']
        self.N_NB=len(self.DFs_NB)            
        self.N_B=len(self.DFs_B)
        self.N_A=len(self.DFs_A)

    def __getitem__(self,i):
        return self.DFs[i]
    def __len__(self):
        return len(self.DFs)

    def __InitFromFile__(self,File, Ucut=None):
        '''
        Init a set of Distribution Functions (RDF,ADF,Potential) from the given rdf or potential file (MagiC 2.0 format).
        Ucut - optional parameter - where to cut prohibiting high part of the potnetial.
        '''        
        import sys
        import DF
        
        Kind=None
        # Stage 1. Read, clean and check
        ifile=open(File,'r')
        lines=ifile.readlines()
        ifile.close()
        # 1.1 Clean lines from comments and leading spaces, convert all to uppercase
        def CleanAndUp(s):
            s=s.strip()
            if s.startswith('&') and not('=' in s): s=s.upper()
            if '=' in s:
                ss=s.split('=')
                ss[0]=ss[0].upper()
                s='='.join(ss)
            return s
        lines = map(CleanAndUp, lines)
        lines = filter(lambda s: (s!='' and (s[0]!='!') and (s[0]!='#')),lines)
        
        # 1.2 Check for header tags, detect Kind
        ## Check that either RDF or Potentials are present in te file
        if (lines.count('&POTENTIAL')>0) & (lines.count('&RDF')>0):
            sys.exit('Error: Both &POTENTIAL and &RDF headers found in the file')    
        
        def checktags(knd,lines):
            if lines.count('&'+knd)>0:
                return knd
            if lines.count('&END'+knd)!=lines.count('&'+knd):        
                sys.exit('Error: Uneven number of '+knd+' and &END'+knd+' tags in the file')    
        Kind=[checktags(x,lines) for x in ('RDF','POTENTIAL') if checktags(x,lines)!=None]
        if len(Kind)!=1: 
            sys.exit('Can not detect what kind of function the file contains: Check the presence of &RDF or &POTENTIAL sections. Convert old rdf/potential file to the new format if neccessary.')
        Kind=Kind[0]
        # Stage 2. Read General block
        ## Check presence of the block
        knd='GENERAL'
        if lines[0]=='&'+knd:
            if lines.count('&'+knd)>1:
                sys.exit('too many &'+knd+' tags in lines')
            if lines.count('&END'+knd)!=1:        
                sys.exit('too many or too few &END'+knd+' tags in lines')
        
        ## Reading properties
        def read_prop(knd,lines, must=True):    
            import re
            query=filter(lambda x: re.search('^'+knd+' *=',x),lines)
            if len(query)>0:
                return query[0].split('=')[1].strip()
            else:
                if must:
                    sys.exit('Error: No '+knd+' value provided')
                else:
                    return None
        # 2.1 Name
#        Name=read_prop('NAME',lines,must=False)
        Name=File
        # 2.2 Number of functions of each type: NB, B, A
        N_NB=int(read_prop('N_NB',lines,must=True))
        N_B=int(read_prop('N_B',lines,must=True))
        N_A=int(read_prop('N_A',lines,must=True))
        # 2.3 Min Max
        Min=float(read_prop('MIN',lines,must=True))
        Max=float(read_prop('MAX',lines,must=True))
        # 2.4 NPoints
        Npoints=int(read_prop('NPOINTS',lines,must=True))
        # 2.5 AtomTypes if reading RDF-file
        AtomTypes=read_prop('TYPES',lines,must=(Kind=='RDF'))
        AtomTypes=AtomTypes.split(',') if AtomTypes!=None else None
        NTypes=read_prop('NTYPES',lines,must=(Kind=='RDF'))
        if NTypes==None:
            NTypes= len(AtomTypes) if AtomTypes!=None else None
        else:
            NTypes=int(NTypes)        
        
        # 3. Include Included files into the list of lines
        def CountIncludes(lines,Kind):
            return sum(map(lambda x: 1 if x.startswith('&INCLUDE'+Kind) else 0, lines))
            
            
        while (CountIncludes(lines,Kind)!=0):
        #3.1 Detect Include line number (with respect to Kind)    
            s_inc=filter(lambda x: x if x.startswith('&INCLUDE'+Kind) else None, lines)[0]# the include directive line
            i_inc=lines.index(s_inc)     # index of the line    
        #3.2 Read included file
            inc_file_name=s_inc.split('=')[1].strip()
            try:        
                inc_file=open(inc_file_name,'r')
                inc_lines=inc_file.readlines()
                inc_file.close()
            except:
                sys.exit('Error: can not read included file '+inc_file_name)         
        # 3.3 Clean and check lines
            inc_lines = map(CleanAndUp, inc_lines)
            inc_lines = filter(lambda s: (s[0]!='!')&(s[0]!='#'),inc_lines)
            lines=lines[0:i_inc]+inc_lines+lines[i_inc+1:] # 3.4 Add included lines to the list
        pass
            
        # 4. Detect &Potential and &RDF blocks
        i_begins= [i for i, s in enumerate(lines) if s == ("&"+Kind)]
        i_ends= [i for i, s in enumerate(lines) if s == ("&END"+Kind)]        
        print '{0:d} {1}-sections found '.format(len(i_begins),Kind)
        # 5. Initializing DF-instancies from sections:        
        DFs=[]    
        for i in xrange(0,len(i_begins)):
            print 'Reading {0}-section {1:d} of {2:d}'.format(Kind,i+1,len(i_begins)),
            # 4.1 Detect type of function inside the block
            Type=read_prop('TYPE',lines[i_begins[i]:i_ends[i]],must=True)
            if (not(Type in ('NB','B','A'))):
                sys.exit('Unknown type stated in lines. Type='+str(Type))
            
            if Kind=='RDF':
                if Type=='NB':
#                    DFs.append(DF.RDF_NB(None,None,None,None,None,Lines=lines[i_begins[i]:i_ends[i]+1]))        
                    DFs.append(DF.RDF_NB(Lines=lines[i_begins[i]:i_ends[i]+1]))        
                elif Type=='B':
#                    DFs.append(DF.RDF_B(None,None,None,None,None,Lines=lines[i_begins[i]:i_ends[i]+1]))        
                    DFs.append(DF.RDF_B(Lines=lines[i_begins[i]:i_ends[i]+1]))        
                    
                elif Type=='A':
#                    DFs.append(DF.RDF_A(None,None,None,None,None,Lines=lines[i_begins[i]:i_ends[i]+1]))        
                    DFs.append(DF.RDF_A(Lines=lines[i_begins[i]:i_ends[i]+1]))        
                pass
            elif Kind=='POTENTIAL':
                if Type=='NB':
                    DFs.append(DF.Pot_NB(None,None,None,None,None,Lines=lines[i_begins[i]:i_ends[i]+1],Ucut=Ucut))        
                elif Type=='B':
                    DFs.append(DF.Pot_B(None,None,None,None,None,Lines=lines[i_begins[i]:i_ends[i]+1]))        
                elif Type=='A':
                    DFs.append(DF.Pot_A(None,None,None,None,None,Lines=lines[i_begins[i]:i_ends[i]+1]))        
                pass

            print 'succesfull!'
        
        
        #Check if  we read everything as it is stated in General section
        if len([x for x in DFs if x.Type=='NB'])!=N_NB:
            sys.exit('Error: Number of read NB-functions ({0:d}) differs from N_NB ({1:d})stated in &General-section'
            .format(len([x for x in DFs if x.Type=='NB']),N_NB))
        if len([x for x in DFs if x.Type=='B'])!=N_B:
            sys.exit('Error: Number of read Bond-functions ({0:d}) differs from N_B ({1:d})stated in &General-section'
            .format(len([x for x in DFs if x.Type=='B']),N_B))
        if len([x for x in DFs if x.Type=='A'])!=N_A:
            sys.exit('Error: Number of read NB-functions ({0:d}) differs from N_A ({1:d})stated in &General-section'
            .format(len([x for x in DFs if x.Type=='A']),N_A))            
        # 6 Initialize object's properties
        self.Name=Name
        self.Kind=Kind
        self.NTypes=NTypes
        if AtomTypes!=None:
            self.Types=AtomTypes
        else:
            def uniq(seq): 
   # order preserving
               noDupes = []
               [noDupes.append(i) for i in seq if not noDupes.count(i)]
               return noDupes
            self.Types=uniq(list([at for DF in DFs if DF.Type=='NB' for at in DF.AtomTypes]))
        self.Min=Min
        self.Max=Max
        self.Npoints=Npoints
        self.DFs=DFs
    def Write(self,ofilename, Split=False):
        '''
            Write the set of functions to the file. 
            If Split=True, all functions will be written in a separate inclde-files.
            If Split=[True, False, True,....] only those functions DFs[i] where Split[i]=True will be written in incude-files.             
        '''
        # 0. Check Split, and convert it to logical list:
        if Split==True:
            Split=[True for i in self.DFs]
        if Split==False:
            Split=[False for i in self.DFs]
        if isinstance(Split, list):
            if len(Split)<len(self.DFs):
                print 'Warning: The Split-list is too short, assume the missing values are False'                
                Split=Split+(len(self.DFs)-len(Split))*[False]
        # 1. Open file for writing
        ofile=open(ofilename,'w')
        # 2. Write &General section
        olines=[]
        olines.append('&General')
        olines.append("NTypes={0:d}".format(self.NTypes))  
        if self.Types!=None:
	    olines.append("Types = "+', '.join(self.Types))           
        olines.append("N_NB={0:d}".format(len(self.DFs_NB)))
        olines.append("N_B={0:d}".format(len(self.DFs_B)))
        olines.append("N_A={0:d}".format(len(self.DFs_A)))
        olines.append("Max={0:8.4f}".format(self.Max))
        olines.append("Min={0:8.4f}".format(self.Min))
        olines.append("NPoints={0:d}".format(self.Npoints))
        olines.append('&EndGeneral')
        olines=map(lambda x: x+'\n',olines)
        ofile.writelines(olines)
        # 3. Write functions in order: NB, B, A
        for i in (self.DFs):
            ofile.write('\n')        
            s=Split[self.DFs.index(i)]
            i._Write(ofile,Split=s)            
        ofile.close()



    def Plot(self,AtOnce=False,hardcopy=False,title='',linetype='',nolegendintra=False,figsize=(10,7),dpi=80):
        '''Plot every function present in the set of functions  (which can be anything of RDF, potential, reference RDF, potential correction).
        AtOnce - if true, all functions are plotted on the same plot (separated by type: i.g. 3 plots are produced: NB, B and A), otherwise each function is plotted separately
        Hardcopy - create eps-copy of the plot,
        linetype - define type of linse using same syntax as matplotlib.pyplot
        nolegendintra - turn off legend on plots with intramolecular DFs. Usually it is usefull then legend overlaps with lines on plot. 
        figsize, dpi - siz eand resolution of the resulting plot in inches and dpi '''
        import matplotlib.pyplot as plt
        def _long_substr_(data):
            substr = ''
            if len(data) > 1 and len(data[0]) > 0:
                for i in range(len(data[0])):
                    for j in range(len(data[0])-i+1):
                        if j > len(substr) and all(data[0][i:i+j] in x for x in data):
                            substr = data[0][i:i+j]
            return substr  
        def _plot_(DFsl,title='', figsize=(10,7), dpi=80, linetype='',nolegendintra=False,hardcopy=False):
            import matplotlib.pyplot as plt        
            
            fig=plt.figure(figsize=figsize,dpi=dpi)        
            plt.hold(False)        
            for i in DFsl:
                if linetype=='':
                    i.Plot(hidden=True,nolegendintra=nolegendintra,noshow=True) 
                else:
                    i.Plot(hidden=True,linetype=linetype,nolegendintra=nolegendintra, noshow=True)
                plt.hold(True)            
            plt.title(title)
            plt.xlabel('r, A')
            plt.legend(loc=0)
            if hardcopy:
                 plt.savefig(title.replace(' ','.')+'.eps')
            plt.show()
            
        if AtOnce:      
            _plot_(self.DFs_NB,title=title+'.{0}.{1}.{2}'.format(self.DFs_NB[0].Name,self.Kind,'NB'),
                        hardcopy=hardcopy,linetype=linetype,nolegendintra=nolegendintra,figsize=figsize,dpi=dpi)        
            if self.DFs_B!=[]:
                _plot_(self.DFs_B,title+'.{0}.{1}.{2}'.format(self.DFs_B[0].Name,self.Kind,'B'),
                        hardcopy=hardcopy,linetype=linetype,nolegendintra=nolegendintra,figsize=figsize,dpi=dpi)
            if self.DFs_A!=[]:
                _plot_(self.DFs_A,title+'.{0}.{1}.{2}'.format(self.DFs_A[0].Name,self.Kind,'A'),
                        hardcopy=hardcopy,linetype=linetype,nolegendintra=nolegendintra,figsize=figsize,dpi=dpi)        
        else:        
            for i in self.DFs:
                fig=plt.figure(figsize=figsize,dpi=dpi)        
                plt.hold(False)        
                i.Plot(hardcopy=hardcopy,linetype=linetype,nolegendintra=nolegendintra,hidden=False, noshow=True)
                plt.title(title+'.{0}.{1}.{2}'.format(i.Name,i.Kind,i.Type))
                plt.hold(True)        
                plt.show()
        return