!    Set of utilities
!
!    RAND   - random number generator
!    RANQ   - generate random quaternion vector
!    ROTATE - rotate a molecule according to a quaternion vector
!    ROTATE2 - rotate a molecule over a given axis on a given angle
!    TAKESTR - read and check a line from the input file
!    RandRotVect - generate random axis for random rotation
!    ReadLineFormText - read line from array of text-lines


module utilsmod
    use precision
contains
!==================== RAND =============================================
!$PRAGMA SUN OPT=0
! Random numbers generator
real(8) function RAND()
    integer(4):: ITWO=2, M2=0,IX=14371,HALFM,IA0,IC0,IA,IC,MIC,IY,M
    real(8) S
    save
    if(M2.eq.0) then
        M=1
        do while (M.gt.M2)
            M2=M
            M=ITWO*M2
        enddo
        HALFM=M2
        IA0=HALFM*datan(1.d0)/8.d0
        IC0=HALFM*(0.5d0-dsqrt(3.d0)/6.d0)
        IA=8*IA0+5
        IC=2*IC0+1
        MIC=(M2-IC)+M2
        S=sngl(0.5d0/HALFM)
    endif
    IY=IX*IA
    if(IY.gt.MIC)IY=(IY-M2)-M2
    IY=IY+IC
    if(IY/2.gt.M2)IY=(IY-M2)-M2
    if(IY.lt.0)IY=(IY+M2)+M2
    IX=IY
    RAND=IY*S
    return
end function rand
!============RanRotVec==========================================
!Generate random vector (to rotate molecule around)
subroutine RanRotVec(x,y,z)
    real(8) x,y,z
    real(8) CosTheta,SinTheta,Phi,Pi

    PI=3.14159256D0
    CosTheta=rand()*2.d0-1.d0
    SinTheta=Dsqrt(1.d0-CosTheta**2)
    Phi=rand()*2.d0*Pi
    x=Dcos(Phi)*sinTheta
    y=Dsin(Phi)*sinTheta
    z=cosTheta
    return
end subroutine RanRotVec
!=============== ROTATE ==============================================
! Rotation of vector (X,Y,Z) using quaternion Q
SUBROUTINE ROTATE(Q,X,Y,Z)
    real(8) Q(4),X,Y,Z
    real(8) SIGN,AA,BB,CC,DD,AB,AC,AD,BC,BD,CD,RX,RY,RZ
!  rotate using quaternion vector Q
    SIGN = -1.D0
    AA   = Q(1)*Q(1)
    BB   = Q(2)*Q(2)
    CC   = Q(3)*Q(3)
    DD   = Q(4)*Q(4)
    AB   = Q(1)*Q(2)
    AC   = Q(1)*Q(3)
    AD   = Q(1)*Q(4)*SIGN
    BC   = Q(2)*Q(3)
    BD   = Q(2)*Q(4)*SIGN
    CD   = Q(3)*Q(4)*SIGN
    RX   = X*(-AA+BB-CC+DD)+Y*(+CD-AB)*2.D0 +Z*(+BC+AD)*2.D0
    RY   = X*(-AB-CD)*2.D0 +Y*(+AA-BB-CC+DD)+Z*(+BD-AC)*2.D0
    RZ   = X*(+BC-AD)*2.D0 +Y*(-AC-BD)*2.D0 +Z*(-AA-BB+CC+DD)
    X    = RX
    Y    = RY
    Z    = RZ
    RETURN
END subroutine ROTATE

!=============ROTATE2=============================================
!Rotate vector (rx,ry,rz) around vector (ax,ay,az) on angle phi using rotation matrix
subroutine Rotate2(rx,ry,rz,phi,ax,ay,az)
    real(8) rx,ry,rz,phi,ax,ay,az
    real(8) CosPhi,OneMinCos,SinPhi,x,y,z,xy,xz,yz,xx,yy,zz,RotMat(3,3)

    CosPhi=Dcos(phi)
    OneMinCos=1-CosPhi
    SinPhi=Dsin(phi)
    xy=ax*ay
    xz=ax*az
    yz=ay*az
    xx=ax*ax
    yy=ay*ay
    zz=az*az
    RotMat(1,1)= CosPhi+OneMinCos*xx
    RotMat(2,1)= OneMinCos*xy+SinPhi*az
    RotMat(3,1)= OneMinCos*xz - SinPhi*ay

    RotMat(1,2)= OneMinCos*xy - SinPhi*az
    RotMat(2,2)= CosPhi + OneMinCos*yy
    RotMat(3,2)= OneMinCos*yz + SinPhi*ax

    RotMat(1,3)= OneMinCos*xz + SinPhi*ay
    RotMat(2,3)= OneMinCos*yz - SinPhi*ax
    RotMat(3,3)= CosPhi + OneMinCos*zz

    x=RotMat(1,1)*rx+RotMat(1,2)*ry+RotMat(1,3)*rz
    y=RotMat(2,1)*rx+RotMat(2,2)*ry+RotMat(2,3)*rz
    z=RotMat(3,1)*rx+RotMat(3,2)*ry+RotMat(3,3)*rz
    rx=x
    ry=y
    rz=z
    return
end subroutine Rotate2

!=============== RANQ =========================================
!
!  Generate random quaternion vector
!$PRAGMA SUN OPT=0
SUBROUTINE RANQ(QP)	
    real(8) QP(4),A,B,C,PI
    PI=3.14159256D0
    A=PI*RAND()/2.D0
    B=PI*RAND()
    C=PI*RAND()
    QP(1)=DSIN(A)*DSIN(B-C)
    QP(2)=DSIN(A)*DCOS(B-C)
    QP(3)=DCOS(A)*DSIN(B+C)
    QP(4)=DCOS(A)*DCOS(B+C)
RETURN
END subroutine RANQ
!
!============== TAKESTR =========================================
!    Read and check a line from input file
!    ----------------------------------------
!
!    This function reads a line from Fortran input file 
!    defined by "KAN" (i.e. opened with parameter unit=KAN)
!    It returns the line if it is not started with # (commentaries); 
!    otherwise it reads and check the next line.
!    Parameter IE:
!    If input value of IE is NOT 99, the subroutine performs normal action
!    (see above) and return IE=0 (normal exit) or IE=-1 (read error)
!    If input value of IE=99, the subroutine is used for diagnostics:
!    it reports the file name and the line number where input error occur.
!
character*128 function TAKESTR(KAN,IE)
!   1 Definitions   
    character*128 AUX, fn*32
    integer(4) IE,KAN
    integer LCOUNT(256)
    data LCOUNT /256*0/
    save AUX,LCOUNT
!   2 Normal action           
    if(IE.eq.99)go to 10
!  LCOUNT variable count line number (including commentaries) 
1   LCOUNT(KAN)=LCOUNT(KAN)+1
    read(KAN,'(a128)',err=10,end=20)AUX
    if(AUX(1:1).eq.'#')go to 1
    TAKESTR=AUX
    IE=0
    return
!   3 Error diagnostic
10  write(*,*)'!!! error in input file '
    if(KAN.eq.5)then
        write(*,*)' Standard input '
    else
        inquire(unit=KAN,name=fn)
        write(*,*)' File ',fn
    end if
    write(*,*)' in line ',LCOUNT(KAN)
    write(*,*)AUX
    stop
!    4 End of file case (is not signaled if input IE < 0 )
20  if(IE.ge.0)then
        write(*,*)'!!! end of file reached'
        if(KAN.eq.5)then
            write(*,*)' Standard input '
        else
            inquire(unit=KAN,name=fn)
            write(*,*)' File ',fn
        end if
!	  stop
    end if
    TAKESTR='  '
    IE=-1
    return
end function takestr

subroutine ReadLineFromText(text,i,line,IE)
    ! the subroutine reads 
    character(len=*) line
    integer IE,i
    character(len=*):: text(:)
    
    line=''
    IE=0      
    do  while (i.le.size(text))        
        if (i.gt.size(text)) then
            ie=1
            return
        endif
        line=TRIM(ADJUSTL(text(i)))
        if ((line(1:1).ne.'#').and.(line(1:1).ne.'!').and.(line(1:1).ne.' ')) then ! The line is not comment
            if (line(len_trim(line):len_trim(line))==',') line=line(1:len_trim(line)-1) ! remove ending comma from line
            return
        else ! Read the next line
            i=i+1
        endif
    enddo
    ie=1
    line=''
    return
end subroutine
end
